#include "dxstdafx.h"
#include "LoadX.hpp"

//** koncerzio miatt
LPWSTR ConvertLPCSTRToLPWSTR(char* pCstring)
{
    LPWSTR pszOut = NULL;
    if (pCstring != NULL)
    {
        int nInputStrLen = (int)strlen(pCstring);
        // Double NULL Termination
        int nOutputStrLen = MultiByteToWideChar(CP_ACP, 0, pCstring, nInputStrLen, NULL, 0) + 2;
        pszOut = new WCHAR [nOutputStrLen];
        if (pszOut)
        {
            memset (pszOut, 0x00, sizeof (WCHAR)*nOutputStrLen);
            MultiByteToWideChar(CP_ACP, 0, pCstring, nInputStrLen, pszOut, nInputStrLen);
        }
    }
    return pszOut;
}

LoadX::LoadX(void) :
	dx(0), dy(0),
	rotX(0), rotY(0),
	rotXdelta(0), rotYdelta(0),
	mesh(0), meshMaterials(0), meshTextures(0), numMaterials(0)
{
}

LoadX::~LoadX(void)
{
}

HRESULT LoadX::onCreateDevice()
{
	D3DXMATRIX m;

	D3DXMatrixPerspectiveFovLH(&m, 3.1415f/4,
		1.0f*this->getWidth()/this->getHeight(),0.0001f,100);
	_device->SetTransform(D3DTS_PROJECTION, &m);

	D3DXMatrixTranslation(&m, 0, -1, 12);
	_device->SetTransform(D3DTS_VIEW, &m);

	setLights();
	loadMesh();

	return S_OK;
}

void LoadX::onDestroyDevice()
{
	SAFE_RELEASE(mesh);
	SAFE_DELETE_ARRAY(meshMaterials);
	for (DWORD i=0; i<numMaterials; ++i)
		SAFE_RELEASE(meshTextures[i]);
	SAFE_DELETE_ARRAY(meshTextures);
}

void LoadX::onFrameMove( double fTime, float fElapsedTime )
{
}

void LoadX::onFrameRender( double fTime, float fElapsedTime )
{
	_device->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(65, 105, 225), 1, 0);

	if ( SUCCEEDED(_device->BeginScene()) )
	{
		for (DWORD i=0; i<numMaterials; ++i)
		{
			//** Anyagjellemzok megadasa
			_device->SetMaterial(&meshMaterials[i]);
			_device->SetTexture(0,meshTextures[i]);

			//** Az al-mesh kirejzolasa
			mesh->DrawSubset( i );
		}

		_device->EndScene();
	}

}

void LoadX::keyboardProc( UINT nChar, bool bKeyDown, bool bAltDown )
{
	switch (nChar)
	{
	case VK_LEFT:
		dx -= 0.05f;
		break;
	case VK_RIGHT:
		dx += 0.05f;
		break;
	case VK_UP:
		dy += 0.05f;
		break;
	case VK_DOWN:
		dy -= 0.05f;
		break;
	}

	D3DXMATRIX m;

	D3DXMatrixTranslation(&m, dx, dy, 0);
	_device->SetTransform(D3DTS_WORLDMATRIX(0), &m);
}

void LoadX::mouseProc(bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown,
				   bool bSideButton1Down, bool bSideButton2Down, INT nMouseWheelDelta,
				   INT xPos, INT yPos )
{
	static int oldX, oldY;
	static bool pressed = false;

	bool changed = false;

	if (bLeftButtonDown)
	{
		if (!pressed)
		{
			pressed = true;
			oldX = xPos;
			oldY = yPos;
		} else {
			rotXdelta = 2.0f*(oldY-yPos)/this->getHeight();
			rotYdelta = 2.0f*(oldX-xPos)/this->getWidth();
			changed = true;
		}
	}
	else
	{
		if (pressed)
		{
			pressed = false;
			rotX += rotXdelta;
			rotY += rotYdelta;
			rotXdelta = 0;
			rotYdelta = 0;
			changed = true;
		}
	}

	if (changed)
	{
		D3DXMATRIX mY, mX, t;

		D3DXMatrixRotationX(&mX, rotX+rotXdelta);
		D3DXMatrixRotationY(&mY, rotY+rotYdelta);
		D3DXMatrixTranslation(&t, 0, -1, 12);
		mX = mY * mX * t;
		_device->SetTransform(D3DTS_VIEW, &mX);
	}
}

void LoadX::setLights()
{
	//** Biztonsag kedveert
	_device->SetRenderState( D3DRS_LIGHTING, TRUE);

	//** Legyen csillanas
//	_device->SetRenderState( D3DRS_SPECULARENABLE, TRUE);

	//** Ambient feny: ahol nem eri direkt feny az objektumokat ott is hat
	_device->SetRenderState( D3DRS_AMBIENT, D3DCOLOR_XRGB(20, 20, 20));

	//---

	//** Irany fenyforras: parhuzamos sugarakban eri a fenye a szinteret
	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(D3DLIGHT9) );

	//** Tipusa
	light.Type    = D3DLIGHT_DIRECTIONAL;
	//** Szine
	light.Diffuse = D3DXCOLOR(1, 1, 1, 1);
	light.Specular = D3DXCOLOR(1, 1, 1, 1);
	light.Range   = 1000.0f;

	//** Iranya
	D3DXVECTOR3 vecDir = D3DXVECTOR3(0, -1, 1);
	D3DXVec3Normalize( (D3DXVECTOR3*)&light.Direction, &vecDir );

	//** Adjuk at a device-nak es engedelyezzuk a feny forrast
	_device->SetLight( 0, &light );
	_device->LightEnable( 0, TRUE );
}

void LoadX::loadMesh()
{
	//** Takaritas, hatha nem eloszor hivjuk ezt
	SAFE_RELEASE(mesh);
	SAFE_DELETE_ARRAY(meshMaterials);
	for (DWORD i=0; i<numMaterials; ++i)
		SAFE_RELEASE(meshTextures[i]);
	SAFE_DELETE_ARRAY(meshTextures);


	LPD3DXBUFFER materialBuffer;
	HRESULT hr;
	
	V( D3DXLoadMeshFromX(L"X\\airplane 2.x", D3DXMESH_MANAGED,
               _device, NULL, &materialBuffer, NULL, &numMaterials,
               &mesh) );


	//** Nem ugyan az, mint a D3DMATERIAL9 !
	D3DXMATERIAL* materials = (D3DXMATERIAL*)materialBuffer->GetBufferPointer();

	//** Tobb anyag es textura is lehet egy mesh-ben
	meshMaterials = new D3DMATERIAL9[numMaterials];
	meshTextures = new LPDIRECT3DTEXTURE9[numMaterials];

	for (DWORD i=0; i<numMaterials; ++i)
	{
		meshMaterials[i] = materials[i].MatD3D;

		//** Az ambiens szint kezzel kell atallitani
		meshMaterials[i].Ambient = meshMaterials[i].Diffuse;

		//** toltsuk be a texturakat, ha vannak
		meshTextures[i] = 0;
		if (materials[i].pTextureFilename)
		{
			LPCWSTR wfilename = ConvertLPCSTRToLPWSTR(materials[i].pTextureFilename);
			WCHAR withpath[60];
			StringCbPrintf(withpath, 60, L"X\\%s", wfilename);

			V( D3DXCreateTextureFromFile(_device, withpath, &meshTextures[i]) );
		}
	}
	
	materialBuffer->Release();
}